@@ -5,7 +5,7 @@ use warnings;
 use Carp;
 use base 'Geo::GDAL::FFI::Object';
 
-our $VERSION = 0.1000;
+our $VERSION = 0.1100;
 
 sub DESTROY {
     my $self = shift;
@@ -86,6 +86,11 @@ sub GetFeature {
     return bless \$f, 'Geo::GDAL::FFI::Feature';
 }
 
+sub GetFeatureCount {
+    my ($self, $force) = @_;
+    Geo::GDAL::FFI::OGR_L_GetFeatureCount($$self, !!$force);
+}
+
 sub SetFeature {
     my ($self, $f) = @_;
     Geo::GDAL::FFI::OGR_L_SetFeature($$self, $$f);
@@ -175,6 +180,13 @@ A set of (vector) features having a same schema (the same Defn
 object). Obtain a layer object by the CreateLayer or GetLayer method
 of a vector dataset object.
 
+Note that the system stores a reference to the parent dataset for
+each layer object to ensure layer objects remain viable.
+If you are relying on a dataset object's destruction to
+flush its dataset cache and then close it then you need to ensure
+all associated child layers are also destroyed.  Failure to do so could
+lead to corrupt data when reading in newly written files.
+
 =head1 METHODS
 
 =head2 GetDefn
@@ -206,6 +218,10 @@ Returns the FeatureDefn object for this layer.
 =head2 DeleteFeature
 
  $layer->DeleteFeature($fid);
+
+=head2 GetFeatureCount
+
+ my $count = $layer->GetFeatureCount();
  
 =head2 GetExtent
  $layer->GetExtent();
